# MODIFICATIONS:
# le 11 Feb 1997 par CARQUEIJAL David : re-modification des procdures :
#   - gred:cmd:save,
#   - gred:cmd:saveas,
#   - gred:cmd:open,
#  Cration des procdures gred:file:... et du fichier grfile.tcl
########################################################################

########################################################################
# gred:file:open --
# Ouvre un fichier dans la fenetre contenant le canvas <I>c</I>.
# Cette procdure ouvre un browser, efface le grafcet prcdent et
# affiche le nouveau.
proc gred:file:open {c} {
   global gred
   upvar #0 grafcet.[gred:getGrafcetName $c] grafcet
   
   $c config -cursor watch
   
   if ![gred:isClean .[gred:getGrafcetName $c]] {
   
      # On va demander confirmation pour "Sauver / Ne pas sauver / Annuler"
      set choice [tk_messageBox -type yesnocancel\
                    -parent .[gred:getGrafcetName $c] \
                    -message "$grafcet(filename) has been modified. \
                              Do you want to save it first?" \
                    -icon warning]
      switch -exact $choice {
        cancel {
            return
        }
        yes {
            gred:file:save $c
        } 
      }

   }
   
   set OK 0
   while {!$OK} {
     set types {
          {{Grafcet Files}   {.gra}}
          {{Text Files}       {.txt}        }
          {{TCL Scripts}      {.tcl}        }
          {{All Files}        *             }
     }
     set fileNameToOpen [tk_getOpenFile -title "Load New File:" \
                                        -defaultextension ".grd" \
                                        -filetypes $types \
                                        -parent .[gred:getGrafcetName $c] \
                                        -initialfile $grafcet(filename)]
     if {![string length $fileNameToOpen]} {
        $c config -cursor tcross
        return
     }
     if {![file readable $fileNameToOpen]} {
        tk_messageBox -type ok \
                -parent .[gred:getGrafcetName $c] \
                -message "File $fileNameToOpen is unreadable ! Try again..." \
                -icon warning
        continue
     }
     set OK 1
   }
  
   # On detruit la structure du grafcet en cours d'edition :
   gred:clear .[gred:getGrafcetName $c]
   # On prpare le chargement du nouveau grafcet
   grafcet:init [gred:getGrafcetName $c]
   
   $c config -cursor tcross
   
   gred:file:load $c $fileNameToOpen
}

########################################################################
# gred:file:load -- Procdure autorisant la lecture "safe" d'un fichier
# Ouvre un fichier dans la fenetre contenant le canvas <I>c</I>.
# Procdure permettant de lire un fichier d'entre contenat un grafcet
proc gred:file:load {c fileName} {
   global gred
   upvar #0 grafcet.[gred:getGrafcetName $c] grafcet
   
   $c config -cursor watch
   
   # On cre un safe interprteur pour loader le fichier
   gred:status $c "Creating safe interp for loading $fileName"

   # initialisation de la liste de objets qui vont etre crees
   Record:clear $c
    
   # On ouvre le fichier en lecture
   set in [open $fileName r]
   
   # On cre l'interprteur qui va permettre de lire le fichier .gra
   interp create -safe loadingInterp
   
   gred:status $c "Loading File : $fileName."
   update
   # On cre les procdures utiles  la cration d'un grafcet
   # Ces procdures vont tre excutes dans l'interprteur matre
   interp alias loadingInterp Etape:create {} Etape:create $c
   interp alias loadingInterp Trans:create {} Trans:create $c
   interp alias loadingInterp Link:create {} Link:create $c
   
   set grafcet(filename) ""
   
   # On value le fichier dans l'interprteur esclave
   if [catch {interp eval loadingInterp [read $in]} error] {
       # Le fichier contient une commande illgale...
       bgerror "$error while reading $fileName"
       # On range nos petits...
       close $in
       interp delete loadingInterp
       # On detruit la structure du grafcet en cours d'dition,
       # on rinitialise gred et on dtruit l'interp...
       gred:clear .[gred:getGrafcetName $c]
       # Ligne suivante a changer ?
       set grafcet(untitled) $gred(untitled)
       gred:markClean .[gred:getGrafcetName $c]
       grafcet:init  [gred:getGrafcetName $c]
       $c config -cursor tcross
       return
   }
   
   # On range nos petits...
   close $in
   interp delete loadingInterp
   
   set grafcet(filename) $fileName
   
   gred:markClean .[gred:getGrafcetName $c]
   undo_Save .[gred:getGrafcetName $c]
   gred:status $c "File : $fileName edited."
   $c config -cursor tcross
}

########################################################################
# gred:file:save --
# Sauvegarde le fichier de la fenetre contenant le canvas <I>c</I>.
# A ce stade, on est certain que le fichier est writable !
# 
# 
proc gred:file:save {c} {
    upvar #0 gred.[gred:getGrafcetName $c] gred
    upvar #0 grafcet.[gred:getGrafcetName $c] grafcet
    
    $c config -cursor watch
    
    if {$grafcet(filename) == ""} {
       gred:file:saveas $c
    } else {
      gred:status $c "Saving File : $grafcet(filename)"
      # on recupere la representation du grafcet sous forme de commandes
      set commands [Obj:getGrafcetCommands $c\
                       [lsort -ascii -increasing [Obj:getAllSelectable $c]]]
      update
      set fid [open $grafcet(filename) w]
      puts $fid \
       "# Fichier sauvegard par Gred, l'diteur de Grafcet dvelopp  l'Ensta"
      puts $fid "# Votre contact  l'Ensta : diam@ensta.fr"
      set clock [clock format [clock seconds] \
                 -format {%d/%m/%y  %T}]
      puts $fid "# Sauvegarde du : $clock"
      puts $fid $commands
      close $fid
      
      gred:markClean .[gred:getGrafcetName $c]
      undo_referenceSet .[gred:getGrafcetName $c]
    }
    
    $c config -cursor tcross
}


########################################################################
# gred:file:saveas --
# Sauvegarde le fichier de la fenetre contenant le canvas <I>c</I>.
# Prpare la sauvegarde d'un fichier en ouvrant un browser de fichier.
# Ne sauvegarde pas le fichier, c'est la procdure <I>gred:file:save</I>
# qui s'en occupe.
proc gred:file:saveas {c} {

   global gred
   upvar #0 grafcet.[gred:getGrafcetName $c] grafcet
   
   $c config -cursor watch
   
   set OK 0
   while {!$OK} {
     set types {
          {{Grafcet Files}   {.gra}}
          {{Text Files}       {.txt}        }
          {{TCL Scripts}      {.tcl}        }
          {{All Files}        *             }
     }
     set fileNameToSave [tk_getSaveFile -title "Save As:" \
                                        -defaultextension ".grd"\
                                        -filetypes $types \
                                        -parent .[gred:getGrafcetName $c] \
                                        -initialfile $grafcet(filename)]
     if ![string length $fileNameToSave] {
         return
     }
     if {[file exists $fileNameToSave] 
         && ![file writable $fileNameToSave]} {
         tk_messageBox -type ok\
            -parent .[gred:getGrafcetName $c] \
            -message "File $fileNameToSave is unwritable ! Try again..." \
            -icon warning
         continue
     }
     set OK 1
   }
   
   set grafcet(filename) $fileNameToSave
   
   gred:updateTitle .[gred:getGrafcetName $c]
   
   $c config -cursor tcross
   
   gred:file:save $c
}

