########################################################################
# Package grant le undo 
# le 14 Feb 1997 par CARQUEIJAL David : Cration du fichier grundo.tcl
########################################################################

########################################################################
# undo_init -- Procdure (re)initialisant une pile d'undo
# Procdure permettant d'initialiser une pile de undo. On doir passer
#  la procdure un identificateur qui permettra reconnatre de faon
# unique chaque pile. 
########################################################################
proc undo_init {undoUid} {
    upvar #0 undo$undoUid undo
    
    if [info exists undo] {
        unset undo
    }
    
    set undo(recordInfo) 0
    set undo(mark) 0

    set undo(uid) 0
    set undo(reference) 0
    return $undoUid
}

########################################################################
# undo_referenceSet -- Permet de poser l'tat de la pile comme rfrence
# Permet de poser l'tat de la pile comme rfrence. Cela est utile
# pour grer les sauvegardes. Par exemple lors de la sauvegarde, on
# indique au package undo que la rfrence est l'tat courant. 
########################################################################
proc undo_referenceSet {undoUid} {
    upvar #0 undo$undoUid undo
    set undo(reference) $undo(uid)
}
########################################################################
# undo_isReference -- Renvoie 1 si l'tat courant est la rfrence
# Permet d'interroger le package undo pour savoir si la pile identifier
# par l'identificateur undoUid est dans un tat dclare comme rfrence
# par la commande <I>undo_referenceSet</I>.
########################################################################
proc undo_isReference {undoUid} {
    upvar #0 undo$undoUid undo
    return [expr $undo(reference) == $undo(uid)]
}

########################################################################
# undo_mark -- Indique le dbut du groupement de commande
# Toutes les commandes qui seront mmorises  l'aide de la procdure
# <I>undo_saveInfos</I> entre un <I>undo_mark</I> et un <I>undo_unMark</I> 
# seront r-excutes ensemble lors d'un undo/redo.
########################################################################
proc undo_mark {undoUid} {
    upvar #0 undo$undoUid undo
    set undo(mark) 1
    
    if [info exists undo(redo,$undo(uid))] {
        unset undo(redo,$undo(uid))
    }
    if [info exists undo(undo,[expr $undo(uid)+1])] {
        unset undo(undo,[expr $undo(uid)+1])
    }
}

########################################################################
# undo_unMark -- Indique la fin du groupement de commande
# Toutes les commandes qui seront mmorises  l'aide de la procdure
# <I>undo_saveInfos</I> entre un <I>undo_mark</I> et un <I>undo_unMark</I> 
# seront r-excutes ensemble lors d'un undo/redo.
########################################################################
proc undo_unMark {undoUid} {
    upvar #0 undo$undoUid undo
    
    if {$undo(mark)} {
        incr undo(uid)
        set undo(mark) 0
        # On efface les infos au dessus dans la pile
        undoCleanPile $undoUid
    }
#     undoPutsRedoInfo $undoUid
#     undoPutsUndoInfo $undoUid
}

########################################################################
# undo_notSave -- Empche la sauvegarde de commande dans la pile
# Parfois il est utile de ne pas sauvegarder de commandes dans la
# pile. C'est possible grce  cette procdure. La sauvegarde de
# commandes par la procdure <I>undo_saveInfos</I> devient inoprante 
# la suite d'un <I>undo_notSave</I>.
########################################################################
proc undo_notSave {undoUid} {
    upvar #0 undo$undoUid undo
    
    set undo(recordInfo) 0
}

########################################################################
# undo_Save -- Rautorise la sauvegarde de commande dans la pile
# Parfois il est utile de ractiv la sauvegarde de commandes dans la
# pile. C'est possible grce  cette procdure. La sauvegarde de
# commandes par la procdure <I>undo_saveInfos</I> redevient active.
########################################################################
proc undo_Save {undoUid} {
    upvar #0 undo$undoUid undo
    
    set undo(recordInfo) 1
}

proc undoCleanPile {undoUid} {
    upvar #0 undo$undoUid undo
    
    set i [expr $undo(uid)+1]
    while {[info exists undo(undo,$i)]} {
        unset undo(undo,$i)
        incr i
    }
    set i $undo(uid)
    while {[info exists undo(redo,$i)]} {
        unset undo(redo,$i)
        incr i
    }
}

########################################################################
# undo_saveInfos -- Permet de sauvegarder des commandes dans la pile des undos
# Permet de sauvegarder une commande pour le undo et une autre pour le redo.
########################################################################
proc undo_saveInfos {undoUid undoI redoI} {
    upvar #0 undo$undoUid undo

    if [set undo(recordInfo)] {
        if [set undo(mark)] {
            # Sauvegarde de multiple commande : Il faut prendre garde des 
            # prcautions :
            # On empile les commandes dans l'ordre de cration pour le redo
            # On empile les commandes dans l'ordre inverse de leur cration
            # pour le undo (logique !)
            if ![info exists undo(redo,$undo(uid))] {
                set undo(redo,$undo(uid)) "$redoI ;"
                set undo(undo,[expr $undo(uid)+1]) "; $undoI"
            } else {
                set undo(redo,$undo(uid)) "$undo(redo,$undo(uid)) $redoI ;"
                set undo(undo,[expr $undo(uid)+1]) \
                                     "; $undoI $undo(undo,[expr $undo(uid)+1])"
            }
        } else {
            # Pas de sauvegarde de multiple commande, cas facile !
            set undo(redo,$undo(uid)) $redoI
            set undo(undo,[incr undo(uid)]) $undoI
            # On efface les infos au dessus dans la pile
            undoCleanPile $undoUid
            
#             undoPutsRedoInfo $undoUid
#             undoPutsUndoInfo $undoUid
        }
    }
}

########################################################################
# undo_undo -- Exccute une commande de undo
# Exccute le undo courant, ie celui point par undo$undoUid(uid)
########################################################################
proc undo_undo {undoUid} {
    upvar #0 undo$undoUid undo
    
    if {$undo(uid) <= 0} {
      gred:status $undoUid "Sorry, no undo informations available !"
      return
    } else {
        gred:status $undoUid "Undoing level $undo(uid)."
        
        set undo(recordInfo) 0
        uplevel #0 $undo(undo,$undo(uid))
        set undo(recordInfo) 1
        incr undo(uid) -1
        Sel:clear [gred:windowToCanvas $undoUid]
        update idletasks
    }
}

########################################################################
# undo_redo -- Exccute une commande de redo
# Exccute le redo courant, ie celui point par undo$undoUid(uid)+1
########################################################################
proc undo_redo {undoUid} {
    upvar #0 undo$undoUid undo
    
    if {![info exists undo(redo,$undo(uid))]} {
      gred:status $undoUid "Sorry, no redo informations available !"
      return
    } else {
        gred:status $undoUid "Redoing level [expr $undo(uid)+1]."
        
        set undo(recordInfo) 0
        uplevel #0 $undo(redo,$undo(uid))
        incr undo(uid)
        set undo(recordInfo) 1
        Sel:clear [gred:windowToCanvas $undoUid]
        update idletasks
    }
}

proc undoPutsRedoInfo {undoUid} {
    upvar #0 undo$undoUid undo
    puts $undo(uid)
    set i 0
    while {[info exists undo(redo,$i)]} {
        puts "redo($i) == $undo(redo,$i)"
        incr i
    }
    puts "-----------------------------------------------"
}

proc undoPutsUndoInfo {undoUid} {
    upvar #0 undo$undoUid undo
    set i 1
    while {[info exists undo(undo,$i)]} {
        puts "undo($i) == $undo(undo,$i)"
        incr i
    }
    puts "-----------------------------------------------"
}